<?php
/**
 * $Horde: whups/admin.php,v 1.84 2006/01/01 21:12:12 jan Exp $
 *
 * Copyright 2002-2006 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 */

@define('WHUPS_BASE', dirname(__FILE__));
require_once WHUPS_BASE . '/lib/base.php';
require_once WHUPS_BASE . '/lib/Admin.php';
require_once 'Horde/Variables.php';

if (!Auth::isAdmin('whups:admin')) {
    Horde::authenticationFailureRedirect();
}

// Set up the page config vars.
$showExtraForm = null;

// Setup vars with all relevant info.
$vars = &Variables::getDefaultVariables();
if (!$vars->exists('action')) {
    $vars->set('action', 'queue');
}

// Admin actions.
$adminurl = Horde::applicationUrl('admin.php');
$tabs = &new Horde_UI_Tabs('action', $vars);
$tabs->addTab(_("Edit Queues"), $adminurl, 'queue');
$tabs->addTab(_("Edit Types"), $adminurl, 'type');
$tabs->addTab(_("Edit Subjects"), $adminurl, 'subject');
$tabs->addTab(_("Queue/Type Matrix"), $adminurl, 'mtmatrix');
$tabs->addTab(_("Send Reminders"), $adminurl, 'reminders');

$renderer = &new Horde_Form_Renderer();

// start the page
function _open($isopened = false)
{
    global $vars;
    static $opened;

    if ($isopened) {
        return $opened;
    }

    if (is_null($opened)) {
        global $registry, $prefs, $browser, $conf, $notification, $title, $tabs;

        $opened = true;
        $title = _("Administration");
        require WHUPS_TEMPLATES . '/common-header.inc';
        require WHUPS_TEMPLATES . '/menu.inc';
        echo $tabs->render($vars->get('action'));
    }
}

switch ($vars->get('formname')) {
case 'addtypestep1form':
    $form1 = &Horde_Form::singleton('AddTypeStep1Form', $vars);
    if ($form1->validate($vars)) {
        // First, add the type
        $tid = $whups->addType($vars->get('name'), $vars->get('description'));
        if (is_a($tid, 'PEAR_Error')) {
            Horde::fatal($tid, __FILE__, __LINE__);
        }

        $vars->add('type', $tid);

        _open();

        $form2 = &Horde_Form::singleton('EditTypeStep2Form', $vars);
        $form2->title = 'addtypestep2form';
        $form2->open($renderer, $vars, 'admin.php', 'post');

        // render the stage 1 form readonly
        $form1->preserve($vars);
        $renderer->beginInactive(sprintf(_("Add Type %s"), _("- Stage 1")));
        $renderer->renderFormInactive($form1, $vars);
        $renderer->end();

        // render the second stage form
        $renderer->beginActive(sprintf(_("Add Type %s"), _("- Stage 2")));
        $renderer->renderFormActive($form2, $vars);
        $renderer->submit();
        $renderer->end();

        $form2->close($renderer);
    } else {
        _open();
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'addtypestep2form':
    $form1 = &Horde_Form::singleton('AddTypeStep1Form', $vars);
    $form2 = &Horde_Form::singleton('EditTypeStep2Form', $vars);
    $form2->_name = 'addtypestep2form';
    break;

case 'edittypestep1form':
    $form1 = &Horde_Form::singleton('EditTypeStep1Form', $vars);
    if ($form1->validate($vars)) {
        switch ($vars->get('submitbutton')) {
        case _("Edit Type"):
            _open();
            $form2 = &Horde_Form::singleton('EditTypeStep2Form', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
            break;

        case _("Delete Type"):
            _open();
            $form2 = &Horde_Form::singleton('DeleteTypeForm', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
            break;

        case _("Clone Type"):
            _open();
            $form2 = &Horde_Form::singleton('CloneTypeForm', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
            break;
        }
    } else {
        _open();
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'clonetypeform':
    $form = &Horde_Form::singleton('CloneTypeForm', $vars);
    if ($form->validate($vars)) {
        // Create a new type and copy all attributes of the clone
        // master to the new type.
        $tid = $vars->get('type');
        $type = $whups->getType($tid);
        $states = $whups->getAllStateInfo($tid);
        $priorities = $whups->getAllPriorityInfo($tid);
        $attributes = $whups->getAttributeInfoFortype($tid);

        // Create the new type.
        $nid = $ntype = $whups->addType($vars->get('name'), $vars->get('description'));

        // Add the states.
        foreach ($states as $s) {
            $whups->addState($nid, $s['state_name'], $s['state_description'], $s['state_category']);
        }

        // Add the priorities.
        foreach ($priorities as $p) {
            $whups->addPriority($nid, $p['priority_name'], $p['priority_description']);
        }

        // Add attributes.
        foreach ($attributes as $a) {
            $whups->addAttributeDesc($nid, $a['attribute_name'], $a['attribute_description']);
        }

        $notification->push(sprintf(_("Successfully Cloned %s to %s."), $type['name'], $vars->get('name')), 'horde.success');
        header('Location: ' . Horde::applicationUrl('admin.php?action=type', true));
        exit;
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'edittypeform':
case 'edittypestep2form':
    $form = &Horde_Form::singleton('EditTypeStep2Form', $vars);
    if ($vars->get('formname') == 'edittypestep2form' && $form->validate($vars)) {
        $result = $whups->updateType($vars->get('type'), $vars->get('name'),
                                     $vars->get('description'));
        if (is_a($result, 'PEAR_Error')) {
            $notification->push(sprintf(_("There was an error modifying the type: %s."), $result->getMessage()), 'horde.error');
        } else {
            $notification->push(sprintf(_("The type \"%s\" has been modified."), $vars->get('name')), 'horde.success');
            _open();
            $form->renderActive($renderer, $vars, 'admin.php', 'post');
        }
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'createdefaultstates':
    $type = $vars->get('type');
    foreach ($conf['states'] as $state) {
        if ($state['active'] == 'active') {
            $whups->addState($type, $state['name'], $state['desc'], $state['category']);
        }
    }

    _open();
    $form = &Horde_Form::singleton('EditTypeStep2Form', $vars);
    $form->renderActive($renderer, $vars, 'admin.php', 'post');
    break;

case 'createdefaultpriorities':
    $type = $vars->get('type');
    foreach ($conf['priorities'] as $priority) {
        if ($priority['active'] == 'active') {
            $whups->addPriority($type, $priority['name'], $priority['desc']);
        }
    }

    _open();
    $form = &Horde_Form::singleton('EditTypeStep2Form', $vars);
    $form->renderActive($renderer, $vars, 'admin.php', 'post');
    break;

case 'deletetypeform':
    $form = &Horde_Form::singleton('DeleteTypeForm', $vars);
    if ($form->validate($vars)) {
        if ($vars->get('yesno') == 1) {
            $result = $whups->deleteType($vars->get('type'));
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push(_("The type has been deleted."), 'horde.success');
            } else {
                $notification->push(sprintf(_("There was an error deleting the type: %s."), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("The type was not deleted."), 'horde.message');
        }
        $vars->set('action', 'type');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'addqueueform':
    $form = &Horde_Form::singleton('AddQueueForm', $vars);
    if ($form->validate($vars)) {
        $result = $whups->addQueue($vars->get('name'), $vars->get('description'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push(sprintf(_("The queue \"%s\" has been created."), $vars->get('name')), 'horde.success');
            _open();

            $vars->set('queue', $result);

            $form2 = &Horde_Form::singleton('EditQueueStep2Form', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
        } else {
            $notification->push(sprintf(_("There was an error creating the queue: %s."), $result->getMessage()), 'horde.error');
            _open();
            $form->renderActive($renderer, $vars, 'admin.php', 'post');
        }
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editqueuestep1form':
    $form1 = &Horde_Form::singleton('EditQueueStep1Form', $vars);
    if ($form1->validate($vars)) {
        switch ($vars->get('submitbutton')) {
        case _("Edit Queue"):
            _open();
            $form2 = &Horde_Form::singleton('EditQueueStep2Form', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
            break;

        case _("Delete Queue"):
            _open();
            $form2 = &Horde_Form::singleton('DeleteQueueForm', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
            break;
        }
    } else {
        _open();
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editqueueform':
case 'editqueuestep2form':
    $form = &Horde_Form::singleton('EditQueueStep2Form', $vars);

    if ($vars->get('formname') == 'editqueuestep2form' && $form->validate($vars)) {
        $result = $whups->updateQueue($vars->get('queue'), $vars->get('name'),
                                      $vars->get('description'), $vars->get('types'),
                                      $vars->get('subjectlist'), $vars->get('subjects'),
                                      $vars->get('versioned'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push(_("The queue has been modified."), 'horde.success');

            if (!$perms->exists('whups:queues:' . $vars->get('queue') . ':update')) {
                $p = &$perms->newPermission('whups:queues:' . $vars->get('queue') . ':update');
                $perms->addPermission($p);
            }
            if (!$perms->exists('whups:queues:' . $vars->get('queue') . ':assign')) {
                $p = &$perms->newPermission('whups:queues:' . $vars->get('queue') . ':assign');
                $perms->addPermission($p);
            }

            _open();
            $form->renderInactive($renderer, $vars);
        } else {
            $notification->push(sprintf(_("There was an error editing the queue: %s."), $result->getMessage()), 'horde.error');
        }
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'deletequeueform':
    $form = &Horde_Form::singleton('DeleteQueueForm', $vars);
    if ($form->validate($vars)) {
        if ($vars->get('yesno') == 1) {
            $result = $whups->deleteQueue($vars->get('queue'));
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push( _("The queue has been deleted."), 'horde.success');
            } else {
                $notification->push(sprintf(_("There was an error deleting the queue: %s."), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("The queue was not deleted."), 'horde.message');
        }
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'addstateform':
    $form = &Horde_Form::singleton('AddStateForm', $vars);
    if ($form->validate($vars)) {
        $result = $whups->addState($vars->get('type'), $vars->get('name'), $vars->get('description'), $vars->get('category'));
        if (!is_a($result, 'PEAR_Error')) {
            $typename = $whups->getType($vars->get('type'));
            $typename = $typename['name'];
            $notification->push(sprintf(_("The state \"%s\" has been added to %s."), $vars->get('name'), $typename), 'horde.success');
        } else {
            $notification->push(sprintf(_("There was an error creating the state: %s."), $result->getMessage()), 'horde.error');
        }

        _open();

        $vars = &new Variables(array('type' => $vars->get('type')));
        $form1 = &Horde_Form::singleton('EditStateStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddStateForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editstatestep1form':
    $form1 = &Horde_Form::singleton('EditStateStep1Form', $vars);
    if (!$vars->get('submitbutton')) {
        _open();
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddStateForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();

        if ($form1->validate($vars)) {
            switch ($vars->get('submitbutton')) {
            case _("Edit State"):
                $form2 = &Horde_Form::singleton('EditStateStep2Form', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;

            case _("Delete State"):
                $form2 = &Horde_Form::singleton('DeleteStateForm', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;
            }
        } else {
            $form1->renderActive($renderer, $vars, 'admin.php', 'post');
        }
    }
    break;

case 'editstatestep2form':
    $form = &Horde_Form::singleton('EditStateStep2Form', $vars);
    if ($form->validate($vars)) {
        $result = $whups->updateState($vars->get('state'), $vars->get('name'),
                                      $vars->get('description'), $vars->get('category'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push(_("The state has been modified."), 'horde.success');

            _open();
            $form->renderInactive($renderer, $vars);
        } else {
            $notification->push(sprintf(_("There was an error editing the state: %s."), $result->getMessage()), 'horde.error');
        }

        _open();

        $vars = &new Variables(array('type' => $vars->get('type')));

        $form1 = &Horde_Form::singleton('EditStateStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddStateForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'deletestateform':
    $form = &Horde_Form::singleton('DeleteStateForm', $vars);
    if ($form->validate($vars)) {
        if ($vars->get('yesno') == 1) {
            $result = $whups->deleteState($vars->get('state'));
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push(_("The state has been deleted."), 'horde.success');
            } else {
                $notification->push(sprintf(_("There was an error deleting the state: %s."), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("The state was not deleted."), 'horde.message');
        }

        _open();

        $form1 = &Horde_Form::singleton('EditStateStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddStateForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'addpriorityform':
    $form = &Horde_Form::singleton('AddPriorityForm', $vars);
    if ($form->validate($vars)) {
        $result = $whups->addPriority($vars->get('type'), $vars->get('name'), $vars->get('description'));
        if (!is_a($result, 'PEAR_Error')) {
            $typename = $whups->getType($vars->get('type'));
            $typename = $typename['name'];
            $notification->push(sprintf(_("The priority \"%s\" has been added to %s."), $vars->get('name'), $typename), 'horde.success');
        } else {
            $notification->push(sprintf(_("There was an error creating the priority: %s."), $result->getMessage()), 'horde.error');
        }

        _open();

        $vars = &new Variables(array('type' => $vars->get('type')));
        $form1 = &Horde_Form::singleton('EditPriorityStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddPriorityForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editprioritystep1form':
    _open();

    $form1 = &Horde_Form::singleton('EditPriorityStep1Form', $vars);
    if (!$vars->get('submitbutton')) {
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddPriorityForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        if ($form1->validate($vars)) {
            switch ($vars->get('submitbutton')) {
            case _("Edit Priority"):
                $form2 = &Horde_Form::singleton('EditPriorityStep2Form', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;

            case _("Delete Priority"):
                $form2 = &Horde_Form::singleton('DeletePriorityForm', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;
            }
        } else {
            $form1->renderActive($renderer, $vars, 'admin.php', 'post');
        }
    }
    break;

case 'editprioritystep2form':
    $form = &Horde_Form::singleton('EditPriorityStep2Form', $vars);
    if ($form->validate($vars)) {
        $result = $whups->updatePriority($vars->get('priority'), $vars->get('name'),
                                         $vars->get('description'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push( _("The priority has been modified."), 'horde.success');

            _open();
            $form->renderInactive($renderer, $vars);
        } else {
            $notification->push(sprintf(_("There was an error editing the priority: %s."), $result->getMessage()), 'horde.error');
        }

        $vars = &new Variables(array('type' => $vars->get('type')));

        _open();
        $form1 = &Horde_Form::singleton('EditPriorityStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddPriorityForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'deletepriorityform':
    $form = &Horde_Form::singleton('DeletePriorityForm', $vars);
    if ($form->validate($vars)) {
        if ($vars->get('yesno') == 1) {
            $result = $whups->deletePriority($vars->get('priority'));
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push(_("The priority has been deleted."), 'horde.success');
            } else {
                $notification->push(sprintf(_("There was an error deleting the priority: %s."), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("The priority was not deleted."), 'horde.message');
        }

        _open();

        $form1 = &Horde_Form::singleton('EditPriorityStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddPriorityForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'adduserform':
    $form = &Horde_Form::singleton('AddUserForm', $vars);
    if ($form->validate($vars)) {
        $info = $whups->getQueue($vars->get('queue'));
        $result = $whups->addQueueUser($vars->get('queue'), $vars->get('user'));
        if (!is_a($result, 'PEAR_Error')) {
            $user = $vars->get('user');
            if (is_array($user)) {
                $userinfo = array();
                foreach ($user as $userID) {
                    $userinfo[] = Whups::formatUser($userID);
                }
                $userinfo = implode(', ', $userinfo);
            } else {
                $userinfo = Whups::formatUser($user);
            }
            $notification->push(sprintf(_("%s added to those responsible for \"%s\""), $userinfo, $info['name']), 'horde.success');
        } else {
            $notification->push(sprintf(_("There was an error adding \"%s\" to the responsible list for \"%s\": %s"), Whups::formatUser($vars->get('user')), $info['name'], $result->getMessage()), 'horde.error');
        }
    }

    _open();

    $form1 = &Horde_Form::singleton('EditUserStep1Form', $vars);
    $form1->renderActive($renderer, $vars, 'admin.php', 'post');
    echo '<br />';

    $vars = &new Variables(array('queue' => $vars->get('queue')));
    $form->renderActive($renderer, $vars, 'admin.php', 'post');
    break;

case 'edituserform':
    $form1 = &Horde_Form::singleton('EditUserStep1Form', $vars);
    $form2 = &Horde_Form::singleton('AddUserForm', $vars);

    _open();

    $form1->renderActive($renderer, $vars, 'admin.php', 'post');
    echo '<br />';

    $vars = &new Variables(array('queue' => $vars->get('queue')));
    $form2 = &Horde_Form::singleton('AddUserForm', $vars);
    $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    break;

case 'edituserstep1form':
    $form = &new EditUserStep1Form($vars);
    if ($form->validate($vars)) {
        $info = $whups->getQueue($vars->get('queue'));
        $result = $whups->removeQueueUser($vars->get('queue'), $vars->get('user'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push(sprintf(_("\"%s\" is no longer among those responsible for \"%s\""), Whups::formatUser($vars->get('user')), $info['name']), 'horde.success');
        } else {
            $notification->push(sprintf(_("There was an error removing \"%s\" from the responsible list for \"%s\": %s"), Whups::formatUser($vars->get('user')), $info['name'], $result->getMessage()), 'horde.error');
        }
    }

    $vars = &new Variables(array('queue' => $vars->get('queue')));
    $form = &new EditUserStep1Form($vars);

    _open();

    $form->renderActive($renderer, $vars, 'admin.php', 'get');

    $form1 = &Horde_Form::singleton('AddUserForm', $vars);
    $form1->renderActive($renderer, $vars, 'admin.php', 'get');
    break;

case 'addsubjectform':
    $form = &Horde_Form::singleton('AddSubjectForm', $vars);
    if ($form->validate($vars)) {
        $result = $whups->addSubject($vars->get('name'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push(sprintf(_("The subject \"%s\" has been added."), $vars->get('name')), 'horde.success');
        } else {
            $notification->push(sprintf(_("There was an error adding the subject: %s."), $result->getMessage()), 'horde.error');
        }
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editsubjectstep1form':
    _open();

    $form1 = &Horde_Form::singleton('EditSubjectStep1Form', $vars);
    if ($form1->validate($vars)) {
        switch ($vars->get('submitbutton')) {
        case _("Edit Subject"):
            $form2 = &Horde_Form::singleton('EditSubjectStep2Form', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
            break;

        case _("Delete Subject"):
            $form2 = &Horde_Form::singleton('DeleteSubjectForm', $vars);
            $form2->renderActive($renderer, $vars, 'admin.php', 'post');
            break;
        }
    } else {
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editsubjectstep2form':
    $form1 = &Horde_Form::singleton('EditSubjectStep2Form', $vars);
    if ($form1->validate($vars)) {
        $result = $whups->updateSubject($vars->get('subject'), $vars->get('name'),
                                        $vars->get('description'), $vars->get('types'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push(_("The subject has been modified."), 'horde.success');

            _open();
            $form1->renderInactive($renderer, $vars);
        } else {
            $notification->push(sprintf(_("There was an error editing the subject: %s."), $result->getMessage()), 'horde.error');
        }
    } else {
        _open();
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'deletesubjectform':
    $form = &Horde_Form::singleton('DeleteSubjectForm', $vars);
    if ($form->validate($vars)) {
        if ($vars->get('yesno') == 1) {
            $result = $whups->deleteSubject($vars->get('subject'));
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push(_("The subject has been deleted."), 'horde.success');
            } else {
                $notification->push(sprintf(_("There was an error deleting the subject: %s."), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("The subject was not deleted."), 'horde.message');
        }
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'addversionform':
    $form = &Horde_Form::singleton('AddVersionForm', $vars);
    if ($form->validate($vars)) {
        $result = $whups->addVersion($vars->get('queue'), $vars->get('name'), $vars->get('description'));
        if (!is_a($result, 'PEAR_Error')) {
            $queuename = $whups->getQueue($vars->get('queue'));
            $queuename = $queuename['name'];
            $notification->push(sprintf(_("The version \"%s\" has been added to %s."), $vars->get('name'), $queuename), 'horde.success');
        } else {
            $notification->push(sprintf(_("There was an error creating the version: %s."), $result->getMessage()), 'horde.error');
        }

        _open();

        $vars = &new Variables(array('queue' => $vars->get('queue')));
        $form1 = &Horde_Form::singleton('EditVersionStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddVersionForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editversionstep1form':
    $form1 = &Horde_Form::singleton('EditVersionStep1Form', $vars);

    _open();

    if (!$vars->get('submitbutton')) {
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddVersionForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        if ($form1->validate($vars)) {
            switch ($vars->get('submitbutton')) {
            case _("Edit Version"):
                $form2 = &Horde_Form::singleton('EditVersionStep2Form', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;

            case _("Delete Version"):
                $form2 = &Horde_Form::singleton('DeleteVersionForm', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;
            }
        } else {
            $form1->renderActive($renderer, $vars, 'admin.php', 'post');
        }
    }
    break;

case 'editversionstep2form':
    $form = &Horde_Form::singleton('EditVersionStep2Form', $vars);
    if ($form->validate($vars)) {
        $result = $whups->updateVersion($vars->get('version'), $vars->get('name'),
                                        $vars->get('description'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push(_("The version has been modified."), 'horde.success');

            _open();
            $form->renderInactive($renderer, $vars);
        } else {
            $notification->push(sprintf(_("There was an error editing the version: %s."), $result->getMessage()), 'horde.error');
        }

        _open();
        $vars = &new Variables(array('queue' => $vars->get('queue')));

        $form1 = &Horde_Form::singleton('EditVersionStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddVersionForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'deleteversionform':
    $form = &Horde_Form::singleton('DeleteVersionForm', $vars);
    if ($form->validate($vars)) {
        if ($vars->get('yesno') == 1) {
            $result = $whups->deleteVersion($vars->get('version'));
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push(_("The version has been deleted."), 'horde.success');
            } else {
                $notification->push(sprintf(_("There was an error deleting the version: %s."), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("The version was not deleted."), 'horde.message');
        }

        _open();

        $form1 = &Horde_Form::singleton('EditVersionStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddVersionForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'addattributedescform':
    $form = &Horde_Form::singleton('AddAttributeDescForm', $vars);
    if ($form->validate($vars)) {
        $result = $whups->addAttributeDesc($vars->get('type'), $vars->get('name'), $vars->get('description'));
        if (!is_a($result, 'PEAR_Error')) {
            $typename = $whups->getType($vars->get('type'));
            $typename = $typename['name'];
            $notification->push(sprintf(_("The attribute \"%s\" has been added to %s."), $vars->get('name'), $typename), 'horde.success');
            $vars = &new Variables(array('type' => $vars->get('type')));
        } else {
            $notification->push(sprintf(_("There was an error creating the attribute: %s."), $result->getMessage()), 'horde.error');
        }

        _open();

        $form1 = &Horde_Form::singleton('EditAttributeDescStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddAttributeDescForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'editattributedescstep1form':
    _open();

    $form1 = &Horde_Form::singleton('EditAttributeDescStep1Form', $vars);
    if (!$vars->get('submitbutton')) {
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddAttributeDescForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        if ($form1->validate($vars)) {
            switch ($vars->get('submitbutton')) {
            case _("Edit Attribute"):
                $form2 = &Horde_Form::singleton('EditAttributeDescStep2Form', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;

            case _("Delete Attribute"):
                $form2 = &Horde_Form::singleton('DeleteAttributeDescForm', $vars);
                $form2->renderActive($renderer, $vars, 'admin.php', 'post');
                break;
            }
        } else {
            $form1->renderActive($renderer, $vars, 'admin.php', 'post');
        }
    }
    break;

case 'editattributedescstep2form':
    $form = &Horde_Form::singleton('EditAttributeDescStep2Form', $vars);
    if ($form->validate($vars)) {
        $result = $whups->updateAttributeDesc($vars->get('attribute'), $vars->get('name'),
                                              $vars->get('description'));
        if (!is_a($result, 'PEAR_Error')) {
            $notification->push( _("The attribute has been modified."), 'horde.success');

            _open();
            $form->renderInactive($renderer, $vars);

            $vars = &new Variables(array('type' => $vars->get('type')));
        } else {
            $notification->push(sprintf(_("There was an error editing the attribute: %s."), $result->getMessage()), 'horde.error');
        }

        _open();

        $form1 = &Horde_Form::singleton('EditAttributeDescStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddAttributeDescForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'deleteattributedescform':
    $form = &Horde_Form::singleton('DeleteAttributeDescForm', $vars);
    if ($form->validate($vars)) {
        if ($vars->get('yesno') == 1) {
            $result = $whups->deleteAttributeDesc($vars->get('attribute'));
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push(_("The attribute has been deleted."), 'horde.success');
            } else {
                $notification->push(sprintf(_("There was an error deleting the attribute: %s."), $result->getMessage()), 'horde.error');
            }
        } else {
            $notification->push(_("The attribute was not deleted."), 'horde.message');
        }

        _open();
        $form1 = &Horde_Form::singleton('EditAttributeDescStep1Form', $vars);
        $form1->renderActive($renderer, $vars, 'admin.php', 'post');

        $form2 = &Horde_Form::singleton('AddAttributeDescForm', $vars);
        $form2->renderActive($renderer, $vars, 'admin.php', 'post');
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'sendreminderform':
    $form = &Horde_Form::singleton('SendReminderForm', $vars);
    if ($form->validate($vars)) {
        $result = Whups::sendReminders($vars);
        if (is_a($result, 'PEAR_Error')) {
            $notification->push($result, 'horde.error');
            _open();
            $form->renderActive($renderer, $vars, 'admin.php', 'post');
        } else {
            $notification->push(_("Reminders were sent."), 'horde.success');
        }
    } else {
        _open();
        $form->renderActive($renderer, $vars, 'admin.php', 'post');
    }
    break;

case 'mtmatrix':
    $vars->set('action', 'mtmatrix');

    $queues = $whups->getQueues();
    $types = $whups->getAllTypes();
    $matrix = $vars->get('matrix');

    // Validate data.
    $pairs = array();
    if (!empty($matrix)) {
        foreach ($matrix as $mid => $mtypes) {
            if (isset($queues[$mid])) {
                foreach ($mtypes as $tid => $on) {
                    if (isset($types[$tid])) {
                        $pairs[] = array($mid, $tid);
                    }
                }
            }
        }
    }

    $result = $whups->updateTypesQueues($pairs);
    if (is_a($result, 'PEAR_Error')) {
        $notification->push($result, 'horde.error');
    } else {
        $notification->push(_("Associations updated successfully."), 'horde.success');
    }
    break;
}

if (!_open(true)) {
    // Check for actions.
    switch ($vars->get('action')) {
    case 'type':
        if (count($whups->getAllTypes())) {
            $main1 = &Horde_Form::singleton('EditTypeStep1Form', $vars);
        }
        $main2 = &Horde_Form::singleton('AddTypeStep1Form', $vars);
        break;

    case 'subject':
        if (count($whups->getSubjects())) {
            $main1 = &Horde_Form::singleton('EditSubjectStep1Form', $vars);
        }
        $main2 = &Horde_Form::singleton('AddSubjectForm', $vars);
        break;

    case 'reminders':
        $main1 = &Horde_Form::singleton('SendReminderForm', $vars);
        break;

    case 'mtmatrix':
        _open();
        $queues = $whups->getQueues();
        $types = $whups->getAllTypes();
        $tlink = Horde::applicationUrl('admin.php?formname=edittypeform');
        $mlink = Horde::applicationUrl('admin.php?formname=editqueueform');
        require WHUPS_TEMPLATES . '/admin/mtmatrix.inc';
        break;

    case 'queue':
        if (count($whups->getQueues())) {
            $main1 = &Horde_Form::singleton('EditQueueStep1Form', $vars);
        }
        if ($registry->hasMethod('tickets/listQueues') == $registry->getApp()) {
            $main2 = &Horde_Form::singleton('AddQueueForm', $vars);
        }
        break;
    }

    _open();
    if (isset($main1)) {
        $main1->renderActive($renderer, $vars, 'admin.php', 'get');
    }
    if (isset($main1) && isset($main2)) {
        echo '<br />';
    }
    if (isset($main2)) {
        $main2->renderActive($renderer, $vars, 'admin.php', 'get');
    }
}

_open();
require $registry->get('templates', 'horde') . '/common-footer.inc';
