<?php
/**
 * Whups_Driver Class.
 *
 * ABC for backends.
 *
 * TODO: Needs updating to include method stubs for all required
 * methods (just to indicate what methods need to be implemented by
 * other backends).
 *
 * $Horde: whups/lib/Driver.php,v 1.148 2006/01/01 21:12:13 jan Exp $
 *
 * Copyright 2001-2006 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 *
 * @author  Robert E. Coyle <robertecoyle@hotmail.com>
 * @package Whups
 */
class Whups_Driver {

    var $_params;

    function Whups_Driver($params)
    {
        $this->_params = $params;
    }

    function setAttributes($info)
    {
        $ticket_id = $info['id'];

        foreach ($info as $name => $value) {
            if ($name[0] == 'a') {
                $this->_setAttributeValue($ticket_id, (int)substr($name, 1), $value);
                $this->updateLog($ticket_id, Auth::getAuth(), array('attribute' => (int)substr($name, 1) . ':' . $value));
            }
        }
    }

    function getHistory($ticket)
    {
        $rows = $this->_getHistory($ticket);
        if (is_a($rows, 'PEAR_Error')) {
            return $rows;
        }

        $history = array();
        foreach ($rows as $row) {
            $transaction = $row['transaction_id'];

            $history[$transaction]['timestamp'] = $row['timestamp'];
            $history[$transaction]['user_id'] = $row['user_id'];
            $history[$transaction]['ticket_id'] = $row['ticket_id'];

            switch ($row['log_type']) {
            case 'comment':
                $history[$transaction]['comment'] = $row['comment_text'];
                $history[$transaction]['changes'][] = array('type' => $row['log_type'],
                                                            'value' => $row['log_value'],
                                                            'comment' => $row['comment_text']);
                break;

            default:
                $label = null;
                switch ($row['log_type']) {
                case 'state':
                    $label = $row['state_name'];
                    break;

                case 'priority':
                    $label = $row['priority_name'];
                    break;

                case 'type':
                    $label = $row['type_name'];
                    break;

                case 'queue':
                    $label = $row['queue_name'];
                    break;
                }

                $history[$transaction]['changes'][] = array('type' => $row['log_type'],
                                                            'value' => $row['log_value'],
                                                            'label' => $label);
            }
        }

        return $history;
    }

    function getQueue($queueId)
    {
        return $GLOBALS['registry']->call('tickets/getQueueDetails', array($queueId));
    }

    function getQueues()
    {
        return $GLOBALS['registry']->call('tickets/listQueues');
    }

    function getVersionInfo($queue)
    {
        return $GLOBALS['registry']->call('tickets/listVersions', array($queue));
    }

    function getVersions($queue)
    {
        if (empty($queue)) {
            return array();
        }

        $versioninfo = $this->getVersionInfo($queue);
        if (is_a($versioninfo, 'PEAR_Error')) {
            return $versioninfo;
        }

        $versions = array();
        foreach ($versioninfo as $vinfo) {
            $versions[$vinfo['id']] = $vinfo['name'];
            if (!empty($vinfo['description'])) {
                $versions[$vinfo['id']] .= ': ' . $vinfo['description'];
            }
        }

        return $versions;
    }

    function getVersion($version)
    {
        return $GLOBALS['registry']->call('tickets/getVersionDetails', array($version));
    }

    /**
     */
    function getCategories()
    {
        return array('unconfirmed' => _("Unconfirmed"),
                     'new' => _("New"),
                     'assigned' => _("Assigned"),
                     'resolved' => _("Resolved"));
    }

    /**
     * Base deleteQueue() implementation. Should be called by driver
     * subclasses after successful removal from the backend. Takes
     * care of cleaning up queue permissions.
     *
     * @param integer $queueId  The id of the queue being deleted.
     */
    function deleteQueue($queueId)
    {
        $perm =& $GLOBALS['perms']->getPermission("whups:queues:$queueId");
        if (is_a($perm, 'PEAR_Error')) {
            return $perm;
        }

        return $perm->delete();
    }

    /**
     */
    function filterTicketsByState($tickets, $state_category = array())
    {
        /* Take a list of tickets and return only those of the
         * specified state_category. */
        $tickets_filtered = array();
        foreach ($tickets as $ticket) {
            foreach ($state_category as $state) {
                if ($ticket['state_category'] == $state) {
                    $tickets_filtered[] = $ticket;
                }
            }
        }

        return $tickets_filtered;
    }

    /**
     * We do some ugly work in here to make sure that no one gets
     * comments mailed to them that they shouldn't see (because of
     * group permissions).
     */
    function mail($ticket_id, $recipients, $subject, $message, $from, $reminder = false)
    {
        global $conf, $registry, $prefs;

        require_once 'Mail.php';
        require_once 'Horde/MIME.php';

        /* Set up recipients and message headers. */
        if (!is_array($recipients)) {
            $recipients = array($recipients);
        }

        $mail_always = null;
        if (!$reminder && !empty($conf['mail']['always_copy'])) {
            $mail_always = $conf['mail']['always_copy'];
            if (strpos($mail_always, '<@>') !== false) {
                $ticket = &Whups_Ticket::makeTicket($ticket_id);
                $mail_always = str_replace('<@>', $ticket->get('queue_name'), $mail_always);
            }
            $headers['To'] = MIME::encodeAddress($mail_always);
            $recipients[] = $mail_always;
        } else {
            $headers['To'] = 'Ticket_Listeners:;';
        }

        if (!empty($conf['mail']['from_addr'])) {
            $headers['From'] = MIME::encodeAddress($conf['mail']['from_addr']);
        } else {
            $headers['From'] = MIME::encodeAddress(Whups::formatUser($from));
        }

        $headers['Subject'] = MIME::encode((is_null($ticket_id) ? '' : '[' . $registry->get('name') . ' #' . $ticket_id . '] ') . $subject, NLS::getCharset());
        $headers['Content-Type'] = 'text/plain; charset=' . NLS::getCharset();

        /* Get our array of comments, sorted in the appropriate
         * order. */
        if (!is_null($ticket_id)) {
            $comments = $this->getHistory($ticket_id);
            if ($conf['mail']['commenthistory'] == 'new' && count($comments)) {
                $comments = array_pop($comments);
                $comments = array($comments);
            } elseif ($conf['mail']['commenthistory'] != 'chronological') {
                $comments = array_reverse($comments);
            }
        } else {
            $comments = array();
        }

        $mails = array();
        $messages = array();
        foreach ($recipients as $user) {
            if (($user == $from && $user == Auth::getAuth()) && $prefs->getValue('email_others_only')) {
                continue;
            }

            /* Make sure to check permissions as a guest for the 'always_copy'
             * address, and as the recipient for all others. */
            if (!empty($mail_always) &&
                $user == $mail_always) {
                $mycomments = Whups::permissionsFilter($comments, 'comment', PERMS_READ, '');
                $key = implode('|', array_keys($mycomments));
                $messages[$key] = str_replace('@@comment_placeholder@@', $this->formatComments($mycomments), $message);
                $mails[$key][] = $mail_always;
            } else {
                $details = Whups::getUserAttributes($user);
                if (!empty($details['email'])) {
                    $email = Whups::formatUser($user);
                    $mycomments = Whups::permissionsFilter($comments, 'comment', PERMS_READ, $user);
                    $key = implode('|', array_keys($mycomments));
                    $messages[$key] = str_replace('@@comment_placeholder@@', $this->formatComments($mycomments), $message);
                    $mails[$key][] = $email;
                }
            }
        }

        /* We may have no recipients due to users excluding themselves
         * from self notifies. */
        if (count($mails)) {
            $mailer = Mail::factory($conf['mailer']['type'], $conf['mailer']['params']);
            foreach ($mails as $key => $addresses) {
                $headers['Bcc'] = MIME::encodeAddress(implode(', ', $addresses));
                $result = $mailer->send(MIME::encodeAddress(implode(', ', $addresses), null, $conf['server']['name']), $headers, $messages[$key]);
            }
        }
    }

    function formatComments($comments)
    {
        $text = '';
        foreach ($comments as $comment) {
            if (!empty($comment['comment_text'])) {
                $text .= "\n";
                $text .= $this->splitMessage(sprintf(_("%s (%s) wrote:"),
                                                     Whups::formatUser($comment['user_id']),
                                                     Horde_Form_Type_date::getFormattedTime($comment['timestamp'], '%Y-%m-%d %H:%M', false)));
                $text .= "\n\n" . $this->splitMessage($comment['comment_text']) . "\n\n\n";
            }
        }

        return $text;
    }

    function splitMessage($message)
    {
        return wordwrap($message, 76, "\n", 1);
    }

    /**
     * Attempts to return a concrete Whups_Driver instance based on $driver.
     *
     * @param string $driver The type of concrete Driver subclass to return.
     * @param array $params  A hash containing any additional configuration or
     *                       connection parameters a subclass might need.
     *
     * @return Whups_Driver  The newly created concrete Whups_Driver instance.
     */
    function &factory($driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['tickets']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('tickets', $driver);
        }

        include_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Whups_Driver_' . $driver;
        if (class_exists($class)) {
            $whups = &new $class($params);
        } else {
            $whups = PEAR::raiseError(sprintf(_("No such backend \"%s\" found"), $driver));
        }

        return $whups;
    }

}
