<?php
/**
 * $Horde: whups/lib/Query.php,v 1.52 2006/01/01 21:12:13 jan Exp $
 *
 * Copyright 2001-2006 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 */

require_once 'Horde/Share.php';
require_once 'Horde/Form/Action.php';

/**
 * Mode types.
 */
define('QUERY_TYPE_AND',       1);
define('QUERY_TYPE_OR',        2);
define('QUERY_TYPE_NOT',       3);
define('QUERY_TYPE_CRITERION', 4);

/**
 * Criterion types.
 */
define('CRITERION_ID',             1);
define('CRITERION_QUEUE',          2);
define('CRITERION_TYPE',           3);
define('CRITERION_STATE',          4);
define('CRITERION_PRIORITY',       5);
define('CRITERION_OWNERS',         7);
define('CRITERION_GROUPS',         9);
define('CRITERION_REQUESTER',      8);
define('CRITERION_ADDED_COMMENT', 11);
define('CRITERION_COMMENT',       12);
define('CRITERION_SUMMARY',       13);
define('CRITERION_ATTRIBUTE',     14);
define('CRITERION_VERSION',       15);

/**
 * Operators for integer fields.
 */
define('OPERATOR_GREATER', 1);
define('OPERATOR_LESS',    2);
define('OPERATOR_EQUAL',   3);

/**
 * Operators for text fields.
 */
define('OPERATOR_CI_SUBSTRING',  4);
define('OPERATOR_CS_SUBSTRING',  5);
define('OPERATOR_WORD',          6);
define('OPERATOR_PATTERN',       7);

$textOperators = array(
    OPERATOR_EQUAL        => _("Exact Match"),
    OPERATOR_CI_SUBSTRING => _("Case Insensitive Substring"),
    // OPERATOR_CS_SUBSTRING => _("Case Sensitive Substring"),
    OPERATOR_WORD         => _("Match Word"),
    OPERATOR_PATTERN      => _("Match Pattern"));

/**
 * array(
 *     'type'      => QUERY_TYPE_...
 *     'children'  => array(...) unless type == QUERY_TYPE_CRITERION
 *     'criterion' => CRITERION_... if  type == QUERY_TYPE_CRITERION
 *     'operator'  => OPERATOR_...  if  type == QUERY_TYPE_CRITERION
 *     'value'     => other argument to operator of criterion
 */

/**
 * @package Whups
 */
class Query extends DataTreeObject_Share {

    function Query($name)
    {
        parent::DataTreeObject_Share($name);
        $this->data = array('parameters' => array(),
                            'perm' => array(),
                            'query' => array(
                                'type' => QUERY_TYPE_AND,
                                'children' => array()));
    }

    /**
     * Parameter form.
     */
    function &getParameterForm(&$vars)
    {
        if (!count($this->data['parameters'])) {
            $result = null;
            return $result;
        }

        $result = &new Horde_Form($vars, _("Query Parameters"), 'queryparameters');

        // You can resubmit this as many times as you like.
        $result->_useFormToken = false;

        foreach ($this->data['parameters'] as $index => $name) {
            $result->addVariable($name, $name, 'text', true);
        }

        return $result;
    }

    /**
     * Tab operations for this query.
     */
    function &getTabs(&$vars)
    {
        if (!$this->datatree) {
            $qManager = &Whups_QueryManager::singleton();
            $this->datatree = &$qManager->getDataTree();
        }

        // Create a few variables that are reused.
        $queryurl = Horde::applicationUrl('query.php');
        $exists = $this->datatree->exists($this);
        $edit = $this->hasPermission(Auth::getAuth(), PERMS_EDIT);
        $delete = $this->hasPermission(Auth::getAuth(), PERMS_DELETE);

        $tabs = &new Horde_UI_Tabs('action', $vars);
        $tabs->addTab(_("New Query"), $queryurl, 'new');
        if (!$exists || $edit) {
            $tabs->addTab(_("Edit Query"), $queryurl, 'edit');
        }
        if ($exists && $edit) {
            Horde::addScriptFile('popup.js', 'horde', true);

            $permsurl = $GLOBALS['registry']->get('webroot', 'horde') . '/services/shares/edit.php';
            $permsurl = Util::addParameter($permsurl, array('app' => 'whups',
                                                            'share' => $this->getName()));
            $tabs->addTab(_("Edit Permissions"), $permsurl, array('tabname' => 'perms',
                                                                  'onclick' => 'popup(\'' . $permsurl . '\'); return false;',
                                                                  'target' => '_blank'));
        }
        $tabs->addTab(_("Execute Query"), Horde::applicationUrl('runquery.php'), 'run');
        $tabs->addTab(_("Load Query"), $queryurl, 'load');
        if ((!$exists && Auth::getAuth()) ||
            ($exists && $edit)) {
            $tabs->addTab(_("Save Query"), $queryurl, 'save');
        }
        if ($exists && $delete) {
            $tabs->addTab(_("Delete Query"), $queryurl, 'delete');
        }

        return $tabs;
    }

    /**
     * Operations.
     */
    function deleteNode($pathstring)
    {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->data['query'];

        if (!strlen($pathstring)) {
            // Deleting the root node isn't supported.
            $GLOBALS['notification']->push(_("Choose New Query instead of deleting the root node."), 'horde.message');
            return false;
        } else {
            $count = count($path) - 1;
            for ($i = 0; $i < $count; $i++) {
                $qobj = &$qobj['children'][$path[$i]];
            }

            if (!empty($qobj['children'][$path[$count]]['value']) &&
                $this->_getParameterName($qobj['children'][$path[$count]]['value']) !== null) {
                unset($this->data['parameters'][array_search($pn, $this->data['parameters'])]);
            }

            array_splice($qobj['children'], $path[$count], 1);
        }
    }

    function hoist($pathstring)
    {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->data['query'];

        if (strlen($pathstring) == 0) {
            // Can't hoist the root node.
        } else {
            $count = count($path) - 1;

            for ($i = 0; $i < $count; $i++) {
                $qobj = &$qobj['children'][$path[$i]];
            }

            $cobj = &$qobj['children'][$path[$count]];

            // TODO: make sure we're hoisting a branch.
            array_splice($qobj['children'], $path[$count], 0, $cobj['children']);
            array_splice($cobj['children'], 0, count($cobj['children']));
        }
    }

    function insertBranch($pathstring, $type)
    {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->data['query'];

        $newbranch = array(
            'type'     => $type,
            'children' => array());

        $count = count($path);

        for ($i = 0; $i < $count; $i++) {
            $qobj = &$qobj['children'][$path[$i]];
        }

        if (!isset($qobj['children']) ||
            !is_array($qobj['children'])) {
            $qobj['children'] = array();
        }

        $qobj['children'][] = $newbranch;
        $path[] = count($qobj['children']) - 1;

        return Query::pathToString($path);
    }

    function insertCriterion($pathstring, $criterion, $cvalue, $operator, $value)
    {
        $path = Query::stringToPath($pathstring);
        $qobj = &$this->data['query'];

        $value = trim($value);
        if ($value[0] == '"') {
            // FIXME: The last character should be '"' as well.
            $value = substr($value, 1, -1);
        } else {
            $pn = $this->_getParameterName($value);
            if ($pn !== null)
                $this->data['parameters'][] = $pn;
        }

        $newbranch = array(
            'type'      => QUERY_TYPE_CRITERION,
            'criterion' => $criterion,
            'cvalue'    => $cvalue,
            'operator'  => $operator,
            'value'     => $value);

        $count = count($path);
        for ($i = 0; $i < $count; $i++) {
            $qobj = &$qobj['children'][$path[$i]];
        }

        $qobj['children'][] = $newbranch;
    }

    /**
     * Top down traversal.
     */
    function walk(&$obj, $method)
    {
        $path = array();
        $more = array();
        $this->_walk($this->data['query'], $more, $path, $obj, $method);
    }

    function _walk(&$node, &$more, &$path, &$obj, $method)
    {
        if ($node['type'] == QUERY_TYPE_CRITERION) {
            $obj->$method($more, $path, QUERY_TYPE_CRITERION, $node['criterion'],
                          $node['cvalue'], $node['operator'], $node['value']);
        } else {
            $obj->$method($more, $path, $node['type'], null, null, null, null);
        }

        if (isset($node['children'])) {
            $count = count($node['children']);

            for ($i = 0; $i < $count; $i++) {
                $path[] = $i;
                $more[] = ($i < $count - 1);
                $this->_walk($node['children'][$i], $more, $path, $obj, $method);
                array_pop($more);
                array_pop($path);
            }
        }
    }

    /**
     * Bottom up traversal.
     */
    function reduce(&$obj, $method, &$vars)
    {
        return $this->_reduce($this->data['query'], $obj, $method, $vars);
    }

    function _reduce(&$node, &$obj, $method, &$vars)
    {
        $args = array();

        if (isset($node['children'])) {
            $count = count($node['children']);

            for ($i = 0; $i < $count; $i++) {
                $result = $this->_reduce($node['children'][$i], $obj, $method, $vars);
                $args[] = $result;
            }
        }

        if ($node['type'] == QUERY_TYPE_CRITERION) {
            $value = $node['value'];

            $pn = $this->_getParameterName($value);
            if ($pn !== null) {
                $value = $vars->get($pn);
            }

            $result = $obj->$method($args, QUERY_TYPE_CRITERION, $node['criterion'],
                                    $node['cvalue'], $node['operator'], $value);
        } else {
            $result = $obj->$method($args, $node['type'], null, null, null, null);
        }

        return $result;
    }

    /**
     * @access private
     */
    function _getParameterName($value)
    {
        if (strcmp(substr($value, 0, 2), '${'))
            return null;

        $pn = substr($value, 2, -1);
        if (!is_string($pn))
            $pn = null;

        return $pn;
    }

    /**
     * @static
     */
    function pathToString(&$path)
    {
        return implode(',', $path);
    }

    /**
     * @static
     */
    function stringToPath($pathstring)
    {
        if (strlen($pathstring) == 0) {
            return array();
        }

        return explode(',', $pathstring);
    }

    /**
     * Gets the ID of this object.
     *
     * @return string  The object's ID.
     */
    function getId()
    {
        if (!$this->datatree) {
            $qManager = &Whups_QueryManager::singleton();
            $this->datatree = &$qManager->getDataTree();
        }

        return parent::getId();
    }

    /**
     * Saves any changes to this object to the backend
     * permanently. New objects are added instead.
     *
     * @return boolean|PEAR_Error  PEAR_Error on failure.
     */
    function save()
    {
        if (!$this->datatree) {
            $qManager = &Whups_QueryManager::singleton();
            $this->datatree = &$qManager->getDataTree();
        }

        return parent::save();
    }

    /**
     * Delete this object from the backend permanently.
     *
     * @return boolean|PEAR_Error  PEAR_Error on failure.
     */
    function delete()
    {
        if (!$this->datatree) {
            $qManager = &Whups_QueryManager::singleton();
            $this->datatree = &$qManager->getDataTree();
        }

        return parent::delete();
    }

    /**
     * Maps this object's attributes from the data array into a format that we
     * can store in the attributes storage backend.
     *
     * @access protected
     *
     * @param boolean $permsonly  Only process permissions? Lets subclasses
     *                            override part of this method while handling
     *                            their additional attributes seperately.
     *
     * @return array  The attributes array.
     */
    function _toAttributes($permsonly = false)
    {
        $originalData = $this->data;

        /* Pre-process attributes to serialize query-specific data. */
        $keys = array_keys($this->data);
        foreach ($keys as $key) {
            if (in_array($key, array('query', 'parameters'))) {
                $this->data[$key] = serialize($this->data[$key]);
            }
        }

        $attributes = parent::_toAttributes($permsonly);
        $this->data = $originalData;
        return $attributes;
    }

    /**
     * Takes in a list of attributes from the backend and maps it to our
     * internal data array.
     *
     * @access protected
     *
     * @param array $attributes   The list of attributes from the backend
     *                            (attribute name, key, and value).
     * @param boolean $permsonly  Only process permissions? Lets subclasses
     *                            override part of this method while handling
     *                            their additional attributes seperately.
     */
    function _fromAttributes($attributes, $permsonly = false)
    {
        parent::_fromAttributes($attributes, $permsonly);

        /* Post-process attributes to unserialize query-specific
         * data. */
        $keys = array_keys($this->data);
        foreach ($keys as $key) {
            if (in_array($key, array('query', 'parameters'))) {
                $this->data[$key] = @unserialize($this->data[$key]);
            }
        }
    }

}

/**
 * @package Whups
 */
class Whups_QueryManager extends Horde_Share {

    /**
     * The subclass of DataTreeObject to instantiate shares as.
     *
     * @var string
     */
    var $_shareObject = 'Query';

    /**
     * Attempts to return a reference to a concrete Whups_QueryManager
     * instance. It will only create a new instance if no Whups_QueryManager
     * instance currently exists.
     *
     * This method must be invoked as:
     *   $var = &Whups_QueryManager::singleton();
     *
     * @return Whups_QueryManager  The concrete QueryManager reference, or
     *                             false on an error.
     */
    function &singleton()
    {
        static $qManager;

        if (!isset($qManager)) {
            $qManager = new Whups_QueryManager('whups');
        }

        return $qManager;
    }

    function &newQuery()
    {
        $query = &$this->newShare(md5(mt_rand()));
        return $query;
    }

}

/**
 * Forms for editing queries.
 *
 * @package Whups
 */
class InsertBranchForm extends Horde_Form {

    function InsertBranchForm(&$vars)
    {
        parent::Horde_Form($vars, _("Insert Branch"));

        $branchtypes = array(
            QUERY_TYPE_AND => _("And"),
            QUERY_TYPE_OR  => _("Or"),
            QUERY_TYPE_NOT => _("Not"));

        $this->addHidden(null, 'path', 'text', false, true);
        $this->addVariable(_("Branch Type"), 'type', 'enum', true, false, null, array($branchtypes));
    }

}

/**
 * @package Whups
 */
class UserCriterionForm extends Horde_Form {

    function UserCriterionForm(&$vars)
    {
        global $textOperators;

        parent::Horde_Form($vars, _("Add User Criterion"));

        $this->addVariable(_("User ID"), 'user', 'text', true);
        $this->addVariable(_("Match Operator"), 'operator', 'enum', true, false, null, array($textOperators));
        $this->addVariable(_("Search Owners"), 'owners', 'boolean', false);
        $this->addVariable(_("Search Requester"), 'requester', 'boolean', false);
        $this->addVariable(_("Search Comments"), 'comments', 'boolean', false);
    }

    function execute(&$vars)
    {
        global $whups_query;

        $path = $vars->get('path');
        $user = $vars->get('user');
        $operator = $vars->get('operator');
        $owners = $vars->get('owners');
        $requester = $vars->get('requester');
        $comments = $vars->get('comments');

        // If we're adding more than one criterion, put them all under
        // an OR node (which should be what is wanted in the general
        // case).
        if ((int)$owners + (int)$requester + (int)$comments > 1) {
            $path = $whups_query->insertBranch($path, QUERY_TYPE_OR);
        }

        if ($owners) {
            $whups_query->insertCriterion($path, CRITERION_OWNERS, null, $operator, $user);
        }

        if ($requester) {
            $whups_query->insertCriterion($path, CRITERION_REQUESTER, null, $operator, $user);
        }

        if ($comments) {
            $whups_query->insertCriterion($path, CRITERION_ADDED_COMMENT, null, $operator, $user);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class GroupCriterionForm extends Horde_Form {

    function GroupCriterionForm(&$vars)
    {
        global $textOperators;

        parent::Horde_Form($vars, _("Add Group Criterion"));

        $groups = &Group::singleton();
        $grouplist = $groups->listGroups();
        if (is_a($grouplist, 'PEAR_Error') || !count($grouplist)) {
            $type_params = array(_("Could not find any groups."));
            $this->addVariable(_("Groups"), 'groups', 'invalid', false, false, null, $type_params);
        } else {
            $this->addVariable(_("Groups"), 'groups', 'enum', true, false, null, array($grouplist));
        }
    }

    function execute(&$vars)
    {
        $path = $vars->get('path');
        $groups = $vars->get('groups');

        if ($groups) {
            $GLOBALS['whups_query']->insertCriterion($path, CRITERION_GROUPS, null, OPERATOR_EQUAL, $groups);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class TextCriterionForm extends Horde_Form {

    function TextCriterionForm(&$vars)
    {
        global $textOperators;

        parent::Horde_Form($vars, _("Add Text Criterion"));

        $this->addVariable(_("Text"), 'text', 'text', true);
        $this->addVariable(_("Match Operator"), 'operator', 'enum', true, false, null, array($textOperators));
        $this->addVariable(_("Search Summary"), 'summary', 'boolean', false);
        $this->addVariable(_("Search Comments"), 'comments', 'boolean', false);
    }

    function execute(&$vars)
    {
        global $whups_query;

        $path = $vars->get('path');
        $text = $vars->get('text');
        $operator = $vars->get('operator');
        $summary = $vars->get('summary');
        $comments = $vars->get('comments');

        if ($summary && $comments) {
            $path = $whups_query->insertBranch($path, QUERY_TYPE_OR);
        }

        if ($summary) {
            $whups_query->insertCriterion($path, CRITERION_SUMMARY, null, $operator, $text);
        }

        if ($comments) {
            $whups_query->insertCriterion($path, CRITERION_COMMENT, null, $operator, $text);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class PropertyCriterionForm extends Horde_Form {

    function PropertyCriterionForm(&$vars)
    {
        global $whups, $whups_query;

        parent::Horde_Form($vars, _("Add Property Criterion"));

        $this->addVariable(_("Id"), 'id', 'intlist', false);

        /* Types. */
        $this->addVariable(_("Type"), 'ttype', 'enum', false, false, null,
                           array($whups->getAllTypes(), _("Any")));


        /* Queues. */
        $queues = Whups::permissionsFilter($whups->getQueues(), 'queue', PERMS_READ);
        if (count($queues)) {
            $action = &Horde_Form_Action::factory('reload');
            $v = &$this->addVariable(_("Queue"), 'queue', 'enum', false, false,
                                     null, array($queues, _("Any")));
            $v->setAction($action);
            if ($vars->get('queue')) {
                $this->addVariable(_("Version"), 'version', 'enum', false,
                                   false, null,
                                   array($whups->getVersions($vars->get('queue')), _("Any")));
            }
        }

        /* States. */
        $states = $whups->getStates();
        $this->addVariable(_("State"), 'state', 'enum', false, false, null,
                           array($states, _("Any")));

        /* Priorities. */
        $priorities = $whups->getPriorities();
        $this->addVariable(_("Priority"), 'priority', 'enum', false, false,
                           null, array($priorities, _("Any")));
    }

    function execute(&$vars)
    {
        global $whups_query;

        $path = $vars->get('path');

        $id = $vars->get('id');
        if (trim($id) != '') {
            $newpath = $path;
            $ids = split("[\\t\\n ,]+", $id);

            if (count($ids) > 1) {
                $newpath = $whups_query->insertBranch($path, QUERY_TYPE_OR);
            }

            foreach ($ids as $id) {
                $whups_query->insertCriterion($newpath, CRITERION_ID, null, OPERATOR_EQUAL, $id);
            }
        }

        $queue = $vars->get('queue');
        if ($queue) {
            $version = $vars->get('version');
            if ($version) {
                $path = $whups_query->insertBranch($path, QUERY_TYPE_AND);
            }
            $whups_query->insertCriterion($path, CRITERION_QUEUE, null, OPERATOR_EQUAL, $queue);
            if ($version) {
                $whups_query->insertCriterion($path, CRITERION_VERSION, null, OPERATOR_EQUAL, $version);
            }
        }

        $type = $vars->get('ttype');
        if ($type) {
            $whups_query->insertCriterion($path, CRITERION_TYPE, null, OPERATOR_EQUAL, $type);
        }

        $state = $vars->get('state');
        if ($state) {
            $whups_query->insertCriterion($path, CRITERION_STATE, null, OPERATOR_EQUAL, $state);
        }

        $priority = $vars->get('priority');
        if ($priority) {
            $whups_query->insertCriterion($path, CRITERION_PRIORITY, null, OPERATOR_EQUAL, $priority);
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class AttributeCriterionForm extends Horde_Form {

    function AttributeCriterionForm(&$vars)
    {
        global $textOperators, $whups, $whups_query;

        parent::Horde_Form($vars, _("Add Attribute Criterion"));

        $type = $whups_query->get('currentTicketType');
        $this->attribs = $whups->getAttributesForType($type);

        if ($this->attribs) {
            $this->addVariable(_("Match"), 'text', 'text', true);
            $this->addVariable(_("Match Operator"), 'operator', 'enum', true, false, null, array($textOperators));

            foreach ($this->attribs as $id => $name) {
                $this->addVariable(sprintf(_("Search %s Attribute"), $name), "a$id", 'boolean', false);
            }
        } else {
            $this->addVariable(_("Search Attribute"), 'attribute', 'invalid', true, false, null, array(_("There are no attributes defined for the current type")));
        }
    }

    function execute(&$vars)
    {
        global $whups_query;

        $path = $vars->get('path');
        $text = $vars->get('text');
        $operator = $vars->get('operator');

        $count = 0;

        foreach ($this->attribs as $id => $name) {
            $count += $vars->exists("a$id") ? 1 : 0;
        }

        if ($count > 1) {
            $path = $whups_query->insertBranch($path, QUERY_TYPE_OR);
        }

        foreach ($this->attribs as $id => $name) {
            if ($vars->get("a$id")) {
                $whups_query->insertCriterion($path, CRITERION_ATTRIBUTE, $id, $operator, $text);
            }
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class ChooseQueryNameForSaveForm extends Horde_Form {

    function ChooseQueryNameForSaveForm(&$vars)
    {
        parent::Horde_Form($vars, _("Save Query"));

        $n = &$this->addVariable(_("Name"), 'name', 'text', true);
        $n->setDefault($GLOBALS['whups_query']->get('name'));
    }

    function execute(&$vars)
    {
        $GLOBALS['whups_query']->set('name', $vars->get('name'));
        $result = $GLOBALS['whups_query']->save();

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class ChooseQueryNameForLoadForm extends Horde_Form {

    function ChooseQueryNameForLoadForm(&$vars)
    {
        global $whups;

        parent::Horde_Form($vars, _("Load Query"));

        $qManager = &Whups_QueryManager::singleton();
        $query_array = $qManager->listShares(Auth::getAuth());
        if (count($query_array)) {
            $qtype = 'enum';
            $qparams = array();
            foreach ($query_array as $id => $query) {
                $qparams[$id] = $query->get('name');
            }
        } else {
            $qtype = 'invalid';
            $qparams = _("You have no saved queries.");
        }

        $this->addVariable(_("Name"), 'name', $qtype, true, false, null, array($qparams));
    }

    function execute(&$vars)
    {
        $qManager = &Whups_QueryManager::singleton();
        $query = $qManager->getShare($vars->get('name'));
        if (is_a($query, 'PEAR_Error')) {
            $GLOBALS['notification']->push(sprintf(_("The query couldn't be loaded:"), $query->getMessage()), 'horde.error');
        } else {
            $GLOBALS['whups_query'] = &$query;
        }

        $this->unsetVars($vars);
    }

}

/**
 * @package Whups
 */
class DeleteQueryForm extends Horde_Form {

    function DeleteQueryForm(&$vars)
    {
        parent::Horde_Form($vars, _("Delete Query?"));

        $yesno = array(array(0 => _("No"), 1 => _("Yes")));
        $this->addVariable(_("Really delete this query? This operation is not undoable."), 'yesno', 'enum', true, false, null, $yesno);
    }

    function execute(&$vars)
    {
        global $whups, $whups_query, $notification;

        if ($vars->get('yesno')) {
            if (!$whups_query->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
                $notifications->push(sprintf(_("Permission denied.")), 'horde.error');
            } else {
                $result = $whups_query->delete();
                if (is_a($result, 'PEAR_Error')) {
                    $notification->push(sprintf(_("The query \"%s\" couldn't be deleted: %s"), $whups_query->get('name'), $result->getMessage()), 'horde.error');
                } else {
                    $notification->push(sprintf(_("The query \"%s\" has been deleted."), $whups_query->get('name')), 'horde.success');
                    $qManager = &Whups_QueryManager::singleton();
                    unset($whups_query);
                    $whups_query = &$qManager->newQuery();
                }
            }
        }

        $this->unsetVars($vars);
    }

}
