--
-- $Horde: whups/scripts/sql/whups.sql,v 1.6 2005/03/20 03:02:56 chuck Exp $
--
-- Copyright 2001-2005 Robert E. Coyle <robertecoyle@hotmail.com>
--
-- See the enclosed file LICENSE for license information (BSD). If you
-- did not receive this file, see http://www.horde.org/licenses/bsdl.php.
--
-- Database definitions for Whups

-- This table only ever contains one row, and the value indicates the
-- version of this database. This allows the code to automatically
-- update the database when a new version of Whups is installed.
CREATE TABLE whups_db_version (
    version             INT NOT NULL
);

INSERT INTO whups_db_version VALUES (1);

CREATE TABLE whups_tickets (
    ticket_id           INT NOT NULL,          -- unique ticket id
    ticket_summary      VARCHAR(255),          -- summary of the ticket
    user_id_requester   VARCHAR(255) NOT NULL, -- user id of the creator of this ticket
    queue_id            INT NOT NULL,          -- queue id that this ticket refers to
    version_id          INT,                   -- version id that this ticket refers to
    type_id             INT NOT NULL,          -- id into the type table, describing the type of ticket
    state_id            INT NOT NULL,          -- state of this ticket, meaning depends on the type of the ticket
    priority_id         INT NOT NULL,          -- priority, meaning depends on the type of the ticket
    ticket_timestamp    INT NOT NULL,          -- redundant but useful, mirrored in the comment and log
--
    PRIMARY KEY (ticket_id)
);

CREATE TABLE whups_ticket_owners (
    ticket_id           INT NOT NULL,
    ticket_owner        VARCHAR(255) NOT NULL
);

CREATE INDEX ticket_id_idx ON whups_ticket_owners (ticket_id);
CREATE INDEX ticket_owner_idx ON whups_ticket_owners (ticket_owner);

CREATE TABLE whups_guests (
    guest_id            VARCHAR(255) NOT NULL,
    guest_email         VARCHAR(255) NOT NULL,
--
    PRIMARY KEY (guest_id)
);

CREATE TABLE whups_queues (
    queue_id            INT NOT NULL,
    queue_name          VARCHAR(64) NOT NULL,
    queue_description   VARCHAR(255),
    queue_subjectlist   SMALLINT DEFAULT 0 NOT NULL,
    queue_versioned     SMALLINT DEFAULT 0 NOT NULL,
--
    PRIMARY KEY (queue_id)
);

CREATE TABLE whups_queues_users (
    queue_id            INT NOT NULL,
    user_uid            VARCHAR(250) NOT NULL,
--
    PRIMARY KEY (queue_id, user_uid)
);

CREATE TABLE whups_subjects (
    subject_id          INT NOT NULL,
    subject_name        VARCHAR(255),
--
    PRIMARY KEY (subject_id)
);

CREATE TABLE whups_queues_subjects (
    queue_id            INT NOT NULL,
    subject_id          INT NOT NULL,
--
    PRIMARY KEY (queue_id, subject_id)
);

CREATE TABLE whups_types (
    type_id             INT NOT NULL,
    type_name           VARCHAR(64) NOT NULL,
    type_description    VARCHAR(255),
--
    PRIMARY KEY (type_id)
);

CREATE TABLE whups_types_queues (
    type_id             INT NOT NULL,
    queue_id            INT NOT NULL
);

CREATE TABLE whups_states (
    state_id            INT NOT NULL,
    type_id             INT NOT NULL,
    state_name          VARCHAR(64) NOT NULL,
    state_description   VARCHAR(255),
    state_category      VARCHAR(16),
--
    PRIMARY KEY (state_id)
);

CREATE TABLE whups_attributes_desc (
    attribute_id          INT NOT NULL,
    type_id               INT NOT NULL,
    attribute_name        VARCHAR(64) NOT NULL,
    attribute_description VARCHAR(255) NOT NULL,
--
    PRIMARY KEY (attribute_id)
);

CREATE TABLE whups_attributes (
    ticket_id           INT NOT NULL,
    attribute_id        INT NOT NULL,
    attribute_value     VARCHAR(255)
);

CREATE TABLE whups_comments (
    comment_id          INT NOT NULL,
    ticket_id           INT NOT NULL,
    user_id_creator     VARCHAR(255) NOT NULL,
    comment_text        TEXT,
    comment_timestamp   INT,
--
    PRIMARY KEY (comment_id)
);

CREATE TABLE whups_logs (
    log_id              INT NOT NULL,
    transaction_id      INT NOT NULL,
    ticket_id           INT NOT NULL,
    log_timestamp       INT NOT NULL,
    log_type            VARCHAR(255) NOT NULL,
    log_value           VARCHAR(255),
    log_value_num       INT,
    user_id             VARCHAR(255) NOT NULL,
--
    PRIMARY KEY (log_id)
);

CREATE INDEX log_transaction_idx ON whups_logs (transaction_id);
CREATE INDEX log_ticket_id_idx ON whups_logs (ticket_id);
CREATE INDEX log_timestamp_idx ON whups_logs (log_timestamp);

CREATE TABLE whups_priorities (
    priority_id           INT NOT NULL,
    type_id               INT NOT NULL,
    priority_name         VARCHAR(64),
    priority_description  VARCHAR(255),
--
    PRIMARY KEY (priority_id)
);

CREATE TABLE whups_versions (
    version_id          INT NOT NULL,
    queue_id            INT NOT NULL,
    version_name        VARCHAR(64),
    version_description VARCHAR(255),
--
    PRIMARY KEY (version_id)
);

CREATE TABLE whups_users_searches (
    user_uid            VARCHAR(255) NOT NULL,
    search_name         VARCHAR(255),
    search_getvars      TEXT
);

CREATE TABLE whups_tickets_listeners (
    ticket_id           INT NOT NULL,
    user_uid            VARCHAR(255) NOT NULL
);

GRANT SELECT, INSERT, UPDATE, DELETE ON whups_db_version TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_tickets TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_guests TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_queues TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_queues_users TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_subjects TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_queues_subjects TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_types TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_types_queues TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_states TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_attributes_desc TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_attributes TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_comments TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_logs TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_priorities TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_versions TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_users_searches TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_tickets_listeners TO horde;
GRANT SELECT, INSERT, UPDATE, DELETE ON whups_ticket_owners TO horde;
