#!/usr/local/bin/php
<?php
/**
 * $Horde: whups/scripts/upgrades/2003-06-25_update_logs_table.php,v 1.1 2004/09/02 19:43:23 chuck Exp $
 *
 * This is a script to update the whups_logs table to add information
 * that's been added recently.
 */

@define('AUTH_HANDLER', true);
@define('HORDE_BASE', dirname(__FILE__) . '/../..');
@define('WHUPS_BASE', HORDE_BASE . '/whups');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

require_once WHUPS_BASE . '/lib/base.php';
require_once WHUPS_BASE . '/lib/Driver.php';

$db = &$whups->_db;
$output = '';

$result = $db->query('SELECT log_type FROM whups_logs LIMIT 1');
if (!is_a($result, 'PEAR_Error')) {
    echo "This table has already been migrated.\n";
    exit;
}

// Is this a very old table format?
$result = $db->query('SELECT type_id_to FROM whups_logs LIMIT 1');
if (is_a($result, 'PEAR_Error')) {
    // Add new fields to whups_logs table
    if ($db->phptype == 'pgsql') {
        $sql = "ALTER TABLE whups_logs ADD COLUMN type_id_to INT;
            UPDATE whups_logs SET type_id_to = 0;
            ALTER TABLE whups_logs ALTER COLUMN type_id_to SET NOT NULL;
            ALTER TABLE whups_logs ALTER COLUMN type_id_to SET DEFAULT 0";
    } else {
        $sql = "ALTER TABLE whups_logs ADD COLUMN type_id_to INT NOT NULL DEFAULT 0";
    }
    $result = $db->query($sql);
    if (is_a($result, 'PEAR_Error')) {
        $output .= "Could not add new column to table whups_logs.\n";
        $output .= $result->toString() . "\n\n";
    }

    if ($db->phptype == 'pgsql') {
        $sql = "ALTER TABLE whups_logs ADD COLUMN module_id_to INT;
            UPDATE whups_logs SET module_id_to = 0;
            ALTER TABLE whups_logs ALTER COLUMN module_id_to SET NOT NULL;
            ALTER TABLE whups_logs ALTER COLUMN module_id_to SET DEFAULT 0";
    } else {
        $sql = "ALTER TABLE whups_logs ADD COLUMN module_id_to INT NOT NULL DEFAULT 0";
    }
    $result = $db->query($sql);
    if (is_a($result, 'PEAR_Error')) {
        $output .= "Could not add new column to table whups_logs.\n";
        $output .= $result->toString() . "\n\n";
    }

    // Fetch all ticket/type id pairs.
    $sql = 'SELECT ticket_id, type_id FROM whups_tickets';
    $ids = $db->getAll($sql, DB_FETCHMODE_ASSOC);
    if (is_a($ids, 'PEAR_Error')) {
        $output .= "Could not create table whups_ticket_owners.\n";
        $output .= $result->toString() . "\n\n";
    } else {
        foreach ($ids as $pair) {
            $sql = 'UPDATE whups_logs SET type_id_to = ' . $pair['type_id'] . ' WHERE ticket_id = ' . $pair['ticket_id'] . ' AND type_id_to = 0 ';
            $result = $db->query($sql);
        }
    }

    $sql = 'INSERT INTO whups_logs (
        ticket_id,
        state_id_to,
        priority_id_to,
        log_timestamp,
        user_id,
        comment_id_log,
        type_id_to,
        module_id_to,
        user_uid_action,
        user_uid_changed
        ) SELECT
        whups_comments.ticket_id,
        whups_tickets.state_id,
        whups_tickets.priority_id,
        whups_comments.comment_timestamp,
        whups_comments.user_id_creator,
        whups_comments.comment_id,
        whups_tickets.type_id,
        whups_tickets.module_id,
        NULL,
        NULL
        FROM
        whups_comments,
        whups_tickets
        WHERE
            whups_tickets.ticket_id = whups_comments.ticket_id
        AND whups_comments.comment_id NOT IN (
            SELECT comment_id_log FROM whups_logs
        );';
    $result = $db->query($sql);
    if (is_a($result,'PEAR_Error')) {
        $output .= 'Could not add log entries for old comments.';
        $output .= $result->toString() . "\n\n";
    }
}

// Add new fields to whups_logs table
$result = $db->query('ALTER TABLE whups_logs ADD COLUMN log_type VARCHAR(255)');
if (is_a($result, 'PEAR_Error')) {
    $output .= "Could not add new column to table whups_logs.\n";
    $output .= $result->toString() . "\n\n";
}

$result = $db->query('ALTER TABLE whups_logs ADD COLUMN log_value VARCHAR(255)');
if (is_a($result, 'PEAR_Error')) {
    $output .= "Could not add new column to table whups_logs.\n";
    $output .= $result->toString() . "\n\n";
}

// Get tickets
$comments = $db->query('SELECT * FROM whups_logs ORDER BY ticket_ID, log_timestamp, comment_id_log');
$queries = array();
$ticket_id = null;
while ($comment = $comments->fetchRow(DB_FETCHMODE_ASSOC)) {
    if ($comment['ticket_id'] != $ticket_id) {
        $states = array();
        $ticket_id = $comment['ticket_id'];
    }
    $processed = false;
    foreach ($comment as $type => $value) {
        if (substr($type, -6) == '_id_to' &&
            (!isset($states[$type]) ||
            $states[$type] != $value)) {
            // Build update query
            $queries[] = 'INSERT INTO whups_logs (ticket_id, log_timestamp, user_id, user_uid_action, user_uid_changed, comment_id_log, log_type, log_value) ' .
                         sprintf('VALUES (%d, %d, %s, %s, %s, %d, %s, %s)',
                                 $comment['ticket_id'],
                                 $comment['log_timestamp'],
                                 $db->quote($comment['user_id']),
                                 $db->quote($comment['user_uid_action']),
                                 $db->quote($comment['user_uid_changed']),
                                 $comment['comment_id_log'],
                                 $db->quote(substr($type, 0, -6)),
                                 $db->quote($comment[$type]));
            $processed = true;
        }
        $states[$type] = $value;
    }
    if (!$processed) {
        $queries[] = 'INSERT INTO whups_logs (ticket_id, log_timestamp, user_id, user_uid_action, user_uid_changed, comment_id_log) ' .
                     sprintf('VALUES (%d, %d, %s, %s, %s, %d)',
                             $comment['ticket_id'],
                             $comment['log_timestamp'],
                             $db->quote($comment['user_id']),
                             $db->quote($comment['user_uid_action']),
                             $db->quote($comment['user_uid_changed']),
                             $comment['comment_id_log']);
    }
}

// Delete old rows
$result = $db->query('DELETE FROM whups_logs');
if (is_a($result,'PEAR_Error')) {
    $output .= "Could not delete old log entries.\n";
    $output .= $result->toString() . "\n\n";
}

// Remove old columns
foreach (array('state_id_to', 'priority_id_to', 'type_id_to', 'module_id_to') as $col) {
    $result = $db->query('ALTER TABLE whups_logs DROP COLUMN ' . $col);
    if (is_a($result, 'PEAR_Error')) {
        $output .= "Could not remove column $col from table whups_logs.\n";
        $output .= $result->toString() . "\n\n";
    }
}

foreach ($queries as $query) {
    $result = $db->query($query);
    if (is_a($result, 'PEAR_Error')) {
        $output .= $result->toString() . "\n\n";
    }
}

if (empty($output)) {
    echo "Migrated successfully.\n";
} else {
    echo $output."\n";
}
