#!/usr/local/bin/php
<?php
/**
 * $Horde: whups/scripts/upgrades/2004-04-05_module_to_queue.php,v 1.6 2006/01/01 21:12:16 jan Exp $
 *
 * Copyright 2004-2006 Jan Schneider <jan@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 */

/* Change this variable to false. */
$live = false;

/*
 * Nothing to see below this line.
 */

@define('AUTH_HANDLER', true);
@define('WHUPS_BASE', dirname(__FILE__) . '/..');

// Load Whups base file.
require_once WHUPS_BASE . '/lib/base.php';

// Need links API to rename module references in links.
require_once 'Horde/Links.php';

// Do CLI checks and environment setup first.
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

if (!$live) {
    echo "TEST MODE. No data will be changed.\nChange the \$live setting in this script to apply the changes.\n\n";
}

// DB setup.
$params = Horde::getDriverConfig('tickets', 'sql');
if (!isset($params['password'])) {
    $params['password'] = '';
}
$dbh = &DB::connect($params);
$dbh->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

// Get the current max id of whups_modules.
$max = $dbh->getOne('SELECT MAX(module_id) FROM whups_modules');
if (is_a($max, 'PEAR_Error')) {
    echo "Fatal error: ", $max->getMessage(), "\n";
    exit(1);
}

// Make sure the sequence is at least that high.
if ($live) {
    echo "Setting whups_queues sequence to $max.\n";
    while ($dbh->nextId('whups_queues') < $max);
} else {
    echo "NOT setting whups_queues sequence to $max.\n";
}

// Get the old modules...
$result = $dbh->query('SELECT module_id, module_name, module_description, module_subjectlist, module_versioned from whups_modules');
if (is_a($result, 'PEAR_Error')) {
    var_dump($result);
    exit(1);
}

// ...and stuff them into the new table.
$sth = $dbh->prepare('INSERT INTO whups_queues (queue_id, queue_name, queue_description, queue_subjectlist, queue_versioned) VALUES (?, ?, ?, ?, ?)');
$count = 0;
while ($row = $result->fetchRow()) {
    if ($live) {
        $exec_result = $dbh->execute($sth, $row);
        if (is_a($exec_result, 'PEAR_Error')) {
            echo "Error '", $exec_result->getMessage(), "' while copying rows to whups_queues table (does it exist?)\n";
            exit(1);
        }
    }
    $count++;
}
if ($live) {
    echo "Copied $count row(s) from whups_modules to whups_queues.\n";
} else {
    echo "NOT copied $count row(s) from whups_modules to whups_queues.\n";
}

// Get the old module subjects...
$result = $dbh->query('SELECT module_id, subject_id from whups_modules_subjects');
if (is_a($result, 'PEAR_Error')) {
    var_dump($result);
    exit(1);
}

// ...and stuff them into the new table.
$sth = $dbh->prepare('INSERT INTO whups_queues_subjects (queue_id, subject_id) VALUES (?, ?)');
$count = 0;
while ($row = $result->fetchRow()) {
    if ($live) {
        $exec_result = $dbh->execute($sth, $row);
        if (is_a($exec_result, 'PEAR_Error')) {
            echo "Error '", $exec_result->getMessage(), "' while copying rows to whups_queues_subjects table (does it exist?)\n";
            exit(1);
        }
    }
    $count++;
}
if ($live) {
    echo "Copied $count row(s) from whups_modules_subjects to whups_queues_subjects.\n";
} else {
    echo "NOT copied $count row(s) from whups_modules_subjects to whups_queues_subjects.\n";
}

// Get the old modules users...
$result = $dbh->query('SELECT module_id, user_uid from whups_modules_users');
if (is_a($result, 'PEAR_Error')) {
    var_dump($result);
    exit(1);
}

// ...and stuff them into the new table.
$sth = $dbh->prepare('INSERT INTO whups_queues_users (queue_id, user_uid) VALUES (?, ?)');
$count = 0;
while ($row = $result->fetchRow()) {
    if ($live) {
        $exec_result = $dbh->execute($sth, $row);
        if (is_a($exec_result, 'PEAR_Error')) {
            echo "Error '", $exec_result->getMessage(), "' while copying rows to whups_queues_users table (does it exist?)\n";
            exit(1);
        }
    }
    $count++;
}
if ($live) {
    echo "Copied $count row(s) from whups_modules_users to whups_queues_users.\n";
} else {
    echo "NOT copied $count row(s) from whups_modules_users to whups_queues_users.\n";
}

// Get the old modules types...
$result = $dbh->query('SELECT type_id, module_id from whups_types_modules');
if (is_a($result, 'PEAR_Error')) {
    var_dump($result);
    exit(1);
}

// ...and stuff them into the new table.
$sth = $dbh->prepare('INSERT INTO whups_types_queues (type_id, queue_id) VALUES (?, ?)');
$count = 0;
while ($row = $result->fetchRow()) {
    if ($live) {
        $exec_result = $dbh->execute($sth, $row);
        if (is_a($exec_result, 'PEAR_Error')) {
            echo "Error '", $exec_result->getMessage(), "' while copying rows to whups_types_queues table (does it exist?)\n";
            exit(1);
        }
    }
    $count++;
}
if ($live) {
    echo "Copied $count row(s) from whups_types_modules to whups_types_queues.\n";
} else {
    echo "NOT copied $count row(s) from whups_types_modules to whups_types_queues.\n";
}

// Fix clients associated with module_id.
$links = &Horde_Links::singleton($registry->getApp());
$link_data['link_params'] = array('link_type' => 'tickets/client',
                                  'from_application' => 'tickets',
                                  'to_application' => 'clients');
$linkData = $links->listLinks($link_data);
$count = 0;
foreach ($linkData as $id => $data) {
    if (!empty($data['from_params']['from_key']) &&
        $data['from_params']['from_key'] == 'module_id') {
        if ($live) {
            $new_data = $data;
            $new_data['from_params']['from_key'] = 'queue_id';
            $ret = $links->addLink($new_data);
            if (is_a($ret, 'PEAR_Error')) {
                echo "Error '", $ret->getMessage(), "' while updating queue/client link.\n";
                exit(1);
            }
            $ret = $links->deleteLink($data);
            if (is_a($ret, 'PEAR_Error')) {
                echo "Error '", $ret->getMessage(), "' while deleting module/client link.\n";
                exit(1);
            }
        }
        $count++;
    }
}

if ($live) {
    echo "Fixed $count queue -> client links.\n";
} else {
    echo "Did NOT fix $count queue -> client links.\n";
}
