<?php
/**
 * $Horde: whups/ticket/people.php,v 1.22 2006/01/01 21:12:16 jan Exp $
 *
 * Copyright 2001-2006 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 */

@define('WHUPS_BASE', dirname(__FILE__) . '/..');
require_once WHUPS_BASE . '/lib/base.php';
require_once WHUPS_BASE . '/lib/Ticket.php';
require_once 'Horde/Variables.php';

$ticket = &Whups::getCurrentTicket();

if (!Auth::getAuth() ||
    !Whups::hasPermission(array($ticket->get('queue')), 'queue',
                          'assign', Auth::getAuth())) {
    $notification->push(_("Permission Denied"), 'horde.error');
    header('Location: ' . Horde::applicationUrl($prefs->getValue('whups_default_view') . '.php', true));
    exit;
}

class AssignTicketForm extends Horde_Form {

    function AssignTicketForm(&$vars, $title = '')
    {
        global $whups, $conf;

        parent::Horde_Form($vars, $title);

        /* Groups. */
        $groups = &Group::singleton();
        if ($conf['prefs']['assign_all_groups']) {
            $mygroups = $groups->listGroups();
        } else {
            $mygroups = $groups->getGroupMemberships(Auth::getAuth());
        }

        $f_users = array();
        $users = $whups->getQueueUsers($vars->get('queue'));
        foreach ($users as $user) {
            $f_users['user:' . $user] = Whups::formatUser($user);
        }

        $f_groups = array();
        if ($mygroups) {
            foreach (array_keys($mygroups) as $id) {
                $f_groups['group:' . $id] = $groups->getGroupName($id, true);
            }
        }

        $this->addHidden('', 'type', 'int', true, true);
        $this->addHidden('', 'id', 'int', true, true);

        if (count($f_users)) {
            asort($f_users);
            $owners = &$this->addVariable(_("Owners"), 'owners', 'multienum', false, false, null, array($f_users));
            $owners->setDefault($whups->getOwners($vars->get('id')));
        }

        if (count($f_groups)) {
            asort($f_groups);
            $group_owners = &$this->addVariable(_("Group Owners"), 'group_owners', 'multienum', false, false, null, array($f_groups));
            $group_owners->setDefault($whups->getOwners($vars->get('id')));
        }

        if (!count($f_users) && !count($f_groups)) {
            $type_params = array(_("There are no users to which this ticket can be assigned."));
            $this->addVariable(_("Owners"), 'owners', 'invalid', false, false, null, $type_params);
        }

        $s = &$this->addVariable(_("New State"), 'tostate', 'enum', true, false, null, array($whups->getStates($vars->get('type'))));
        $s->setDefault($vars->get('state'));

        $this->addVariable(_("Comment"), 'newcomment', 'longtext', false);

        /* Group restrictions. */
        $groups = &Group::singleton();
        $mygroups = $groups->getGroupMemberships(Auth::getAuth());

        if ($mygroups && Auth::isAdmin('whups:admin') ||
            $GLOBALS['perms']->hasPermission('whups:hiddenComments', Auth::getAuth(), PERMS_EDIT)) {
            foreach (array_keys($mygroups) as $gid) {
                $grouplist[$gid] = $groups->getGroupName($gid, true);
            }
            asort($grouplist);
            $grouplist = array_merge(array(0 => _("Any Group")), $grouplist);
            $this->addVariable(_("Viewable only by members of"), 'group', 'enum', true, false, null, array($grouplist));
        }
    }

}

$vars = &Variables::getDefaultVariables();
$vars->set('id', $id = $ticket->getId());
foreach ($ticket->getDetails() as $varname => $value) {
    $vars->add($varname, $value);
}

// Assign Ticket action.
if ($vars->get('formname') == 'assignticketform') {
    $assignform = &new AssignTicketForm($vars);
    if ($assignform->validate($vars)) {
        $assignform->getInfo($vars, $info);

        $ticket->change('owners', array_merge(isset($info['owners']) ? $info['owners'] : array(),
                                              isset($info['group_owners']) ? $info['group_owners'] : array()));
        $ticket->change('state', $info['tostate']);

        // Add comment if present.
        if (!empty($info['newcomment'])) {
            $ticket->change('comment', $info['newcomment']);
        }

        // If there was a new comment and permissions were specified
        // on it, set them.
        if (!empty($info['group'])) {
            $ticket->change('comment-perms', $info['group']);
        }

        $result = $ticket->commit();
        if (is_a($result, 'PEAR_Error')) {
            $notification->push($result, 'horde.error');
        } else {
            $notification->push(_("Assign Ticket Succeeded"), 'horde.success');
            $ticket->show();
        }
    }
}

$title = sprintf(_("Update People for %s"), '[#' . $id . '] ' . $ticket->get('summary'));
require WHUPS_TEMPLATES . '/common-header.inc';
require WHUPS_TEMPLATES . '/menu.inc';
require WHUPS_TEMPLATES . '/prevnext.inc';

$tabs = &Whups::getTicketTabs($vars);
$tabs->preserve('id', $id);
echo $tabs->render();

$r = &new Horde_Form_Renderer();

$form = &new AssignTicketForm($vars, $title);
$form->renderActive($r, $vars, 'people.php', 'post');
echo '<br />';

$ticket->setDetails($vars);
$form = &new TicketDetailsForm($vars, '[#' . $id . '] ' . $ticket->get('summary'));
$form->addAttributes($whups->getAllTicketAttributesWithNames($id));
$form->renderInactive($r, $vars);

require $registry->get('templates', 'horde') . '/common-footer.inc';
