<?php
/**
 * $Horde: whups/ticket/update.php,v 1.23 2006/01/01 21:12:16 jan Exp $
 *
 * Copyright 2001-2006 Robert E. Coyle <robertecoyle@hotmail.com>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 */

@define('WHUPS_BASE', dirname(__FILE__) . '/..');
require_once WHUPS_BASE . '/lib/base.php';
require_once WHUPS_BASE . '/lib/Ticket.php';
require_once 'Horde/Variables.php';
require_once 'Text/Flowed.php';

$ticket = &Whups::getCurrentTicket();

if (!Auth::getAuth() ||
    !Whups::hasPermission(array($ticket->get('queue')), 'queue',
                          'update', Auth::getAuth())) {
    $notification->push(_("Permission Denied"), 'horde.error');
    header('Location: ' . Horde::applicationUrl($prefs->getValue('whups_default_view') . '.php', true));
    exit;
}

/**
 * Basic Ticket Editing forms.
 *
 * @author  Chuck Hagenbuch <chuck@horde.org>
 * @package Whups
 */
class EditForm extends Horde_Form {

    function EditForm(&$vars, $title = '', $attributes = null)
    {
        global $whups;

        parent::Horde_Form($vars, $title);
        $type = $vars->get('type');

        $this->addHidden('', 'id', 'int', true, true);
        $this->addHidden('', 'type', 'int', true, true);

        /* Summary. */
        $this->addVariable(_("Summary"), 'summary', 'text', true);

        /* State. */
        $this->addVariable(_("State"), 'state', 'enum', true, false, null, array($whups->getStates($type, '', 'unconfirmed')));

        /* Priority. */
        $this->addVariable(_("Priority"), 'priority', 'enum', true, false, null, array($whups->getPriorities($type)));

        /* Ticket Attributes. */
        $attribute_array = $whups->getAttributesForType($type);
        foreach ($attribute_array as $id => $name) {
            $var = &$this->addVariable($name, "a$id", 'text', false);
            if (isset($attributes[$name])) {
                $var->setDefault($attributes[$name]);
            }
        }

        /* Attachments and Comments. */
        $this->addVariable(_("Attachment"), 'newattachment', 'file', false);
        $this->addVariable(_("Comment"), 'newcomment', 'longtext', false);

        /* Comment Permissions. */
        $groups = &Group::singleton();
        $mygroups = $groups->getGroupMemberships(Auth::getAuth());
        if ($mygroups) {
            foreach (array_keys($mygroups) as $gid) {
                $grouplist[$gid] = $groups->getGroupName($gid, true);
            }
            asort($grouplist);
            $grouplist = array_merge(array(0 => _("Any Group")), $grouplist);
            $this->addVariable(_("Restrict Comment to:"), 'group', 'enum', true, false, null, array($grouplist));
        }
    }

    function validate(&$vars)
    {
        if (!Auth::getAuth()) {
            $this->setError('_auth', _("Permission Denied."));
        }

        return parent::validate($vars);
    }

}

$vars = &Variables::getDefaultVariables();
$vars->set('id', $id = $ticket->getId());
foreach ($ticket->getDetails() as $varname => $value) {
    $vars->add($varname, $value);
}
if ($tid = $vars->get('transaction')) {
    $history = Whups::permissionsFilter($whups->getHistory($ticket->getId()),
                                        'comment', PERMS_READ);
    if (!empty($history[$tid]['comment'])) {
        $flowed = &new Text_Flowed(preg_replace("/\s*\n/U", "\n", $history[$tid]['comment']));
        $vars->set('newcomment', $flowed->toFlowed(true));
    }
}

// Edit action.
if ($vars->get('formname') == 'editform') {
    $editform = &new EditForm($vars);
    if ($editform->validate($vars)) {
        $editform->getInfo($vars, $info);

        $ticket->change('summary', $info['summary']);
        $ticket->change('state', $info['state']);
        $ticket->change('priority', $info['priority']);
        if (!empty($info['newcomment'])) {
            $ticket->change('comment', $info['newcomment']);
        }

        // Update attributes.
        $whups->setAttributes($info);

        // Add attachment if one was uploaded.
        if (!empty($info['newattachment']['name'])) {
            $ticket->change('attachment', array('name' => $info['newattachment']['name'],
                                                'tmp_name' => $info['newattachment']['tmp_name']));
        }

        // If there was a new comment and permissions were specified
        // on it, set them.
        if (!empty($info['group'])) {
            $ticket->change('comment-perms', $info['group']);
        }

        $result = $ticket->commit();
        if (is_a($result, 'PEAR_Error')) {
            $notification->push($result, 'horde.error');
        } else {
            $notification->push(_("Ticket Updated"), 'horde.success');
            $ticket->show();
        }
    }
}

$title = '[#' . $id . '] ' . $ticket->get('summary');
require WHUPS_TEMPLATES . '/common-header.inc';
require WHUPS_TEMPLATES . '/menu.inc';
require WHUPS_TEMPLATES . '/prevnext.inc';

$tabs = &Whups::getTicketTabs($vars);
$tabs->preserve('id', $id);
echo $tabs->render();

$r = &new Horde_Form_Renderer();
$form = &new EditForm($vars, sprintf(_("Update %s"), $title), $whups->getAllTicketAttributesWithNames($id));
$form->renderActive($r, $vars, 'update.php', 'post');
echo '<br class="spacer" />';

$ticket->setDetails($vars);
$form = &new TicketDetailsForm($vars, $title);
$form->addAttributes($whups->getAllTicketAttributesWithNames($id));
$form->renderInactive($r, $vars);

require $registry->get('templates', 'horde') . '/common-footer.inc';
