<?php
/**
 * $Horde: whups/view.php,v 1.16 2006/01/01 21:12:13 jan Exp $
 *
 * Copyright 2003-2006 Jan Schneider <jan@horde.org>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/licenses/bsdl.php.
 */

define('WHUPS_BASE', dirname(__FILE__));
require_once WHUPS_BASE . '/lib/base.php';
require_once 'Horde/MIME/Magic.php';
require_once 'Horde/MIME/Viewer.php';
require_once 'Horde/MIME/Part.php';
require HORDE_BASE . '/config/mime_drivers.php';
require WHUPS_BASE . '/config/mime_drivers.php';

$actionID = Util::getFormData('actionID');
$ticket = Util::getFormData('ticket');
$filename = Util::getFormData('file');
$type = Util::getFormData('type');

// Get the ticket details first.
$details = $whups->getTicketDetails($ticket);
if (is_a($details, 'PEAR_Error')) {
    if ($details->code === 0) {
        // No permissions to this ticket.
        $url = Horde::url($registry->get('webroot', 'horde') . '/login.php', true);
        $url = Util::addParameter($url, 'url', Horde::selfUrl(true));
        header('Location: ' . $url);
        exit;
    } else {
        Horde::fatal($details->getMessage(), __FILE__, __LINE__);
    }
}

// Check permissions on this ticket.
if (!count(Whups::permissionsFilter(array($details['queue'] => ''), 'queue', PERMS_READ))) {
    Horde::fatal(_("You are not allowed to view this ticket."), __FILE__, __LINE__);
}

if (empty($conf['vfs']['type'])) {
    Horde::fatal(_("The VFS backend needs to be configured to enable attachment uploads."), __FILE__, __LINE__);
}

require_once 'VFS.php';
$vfs = &VFS::singleton($conf['vfs']['type'], Horde::getDriverConfig('vfs'));
if (is_a($vfs, 'PEAR_Error')) {
    Horde::fatal($vfs, __FILE__, __LINE__);
} else {
    $data = $vfs->read(WHUPS_VFS_ATTACH_PATH . '/' . $ticket, $filename);
}
if (is_a($data, 'PEAR_Error')) {
    Horde::fatal(sprintf(_("Access denied to %s"), $filename), __FILE__, __LINE__);
}

/* Run through action handlers */
switch ($actionID) {
case 'download_file':
     $browser->downloadHeaders($filename);
     echo $data;
     exit;

case 'view_file':
    $mime_part = &new MIME_Part(MIME_Magic::extToMIME($type), $data);
    $mime_part->setName($filename);
    $viewer = &MIME_Viewer::factory($mime_part);

    $body = $viewer->render();
    $browser->downloadHeaders($mime_part->getName(), $viewer->getType(), true, strlen($body));
    echo $body;
    exit;
}
